function [totalb,totalc,savings,totalenergy,maxenergy] = testdynamicenergysavings(trafficp,lambda_max,mu,delta)
%Discrete event simulation on 49-cell wrapped-around model to test energy
%savings under dynamic BS (base station) sleeping scheme;
%Adjust the input parameter "trafficp" to see the impacts of different
%assumptions that how much energy consumption is dependent on carried
%traffic in BSs

%Power consumption of a BS can be broken into three parts:
%1. basic operation of a BS, such as air conditioning and
%power supply, consumed when the BS is active
%2. signal processing to coordinate other BS in the network, consumed when
%the BS is active
%3. power amplifier to connect to the mobile users, consumed proportionally
%with carried traffic of the BS

%output parameters
%totalb: number of blocked calls
%totalc: number of calls initiated
%savings (= totalenergy/maxenergy): relative energy savings as compared to the base case where no BS
%sleeping is implemented
%lambda_max: peak arrival rate
%mu: service rate
%delta: handover rate
signaling = 0.5; %proportion of energy consumption by power amplifier AND signal processing in total energy consumption of a BS
cooling = 1-signaling; %proportion of energy consumption by basic operations in total energy consumption of a BS
controlp = 1-trafficp; %proportion of energy consumption by signal processing in "signaling" (while trafficp is the proportion of energy consumption by power amplifier in "signaling")

[a,b]=textread('lambda.dat','%f %f'); %read the traffic profile, replace the file to get results under different traffic profiles
totalenergy=0;
maxenergy=0;
N=49;
K=20;

Q=zeros(49,1);
A=zeros(49,1);
B=zeros(49,1);

Status=ones(1,49); %vector to indicate whether the corresponding BS is active or sleeping
Neis=neighbors(); %set up the layout of the 49-cell model
x=mu; 
z=delta;
y=lambda_max;
t=0;



laston=49;
lasttime=0;
while t<1600
    i=1;
    while i<length(a),
        if t>=a(i)/6.2832*1600&&t<a(i+1)/6.2832*1600,
            y=30*((b(i+1)-b(i))/(a(i+1)-a(i))*t*6.2832/1600+(a(i+1)*b(i)-a(i)*b(i+1))/(a(i+1)-a(i)));
            break;
        end
        i=i+1;
    end

    r(1)=-1/y/N*log(1-rand());      
    nt(1)=t+r(1);

    if sum(Q)>0 
            r(2)=-1/x/sum(Q)*log(1-rand());            
            nt(2)=t+r(2);
            r(3)=-1/z/sum(Q)*log(1-rand());           
            nt(3)=t+r(3);
    else
       
        r(2)=-1/x*log(1-rand());      
        nt(2)=nt(1)+r(2);
        r(3)=-1/z*log(1-rand());
        nt(3)=nt(1)+r(3);
        
    end

    event=min(nt);

    if event==1, %arrival of a call
        totalenergy=totalenergy+sum(Q)*(nt(1)-t)*850*signaling*trafficp/10*54+(laston*(850*cooling)+laston*850*signaling*controlp)*54*(nt(1)-t);
        maxenergy = maxenergy +sum(Q)*(nt(1)-t)*850*signaling*trafficp/10*54+(49*(850*cooling)+49*850*signaling*controlp)*54*(nt(1)-t);

        i=1;
        ran=rand();
        while ran>i/N,
            i=i+1;
        end
        if Status(i)~=0,
            A(i)=A(i)+1;
            if Q(i)<K,
                Q(i)=Q(i)+1;
            else
                B(i)=B(i)+1;
            end
        else
            count3=0;
            
            temp=zeros(1,6);
            j=1;
            m=1;
            while j<=6,
                if Status(Neis(i,j))~=0,
                    count3=count3+1;
                    temp(m)=Neis(i,j);
                    m=m+1;
                end
                
                j=j+1;
            end
            if count3>0,
                j=1;
                ran=rand();
                while ran>j/count3,
                    j=j+1;
                end
                A(temp(j))=A(temp(j))+1;
                if Q(temp(j))<K,
                    Q(temp(j))=Q(temp(j))+1;
                else
                    B(temp(j))=B(temp(j))+1;
                end
            else
                B(i)=B(i)+1;
            end
        end
        
        t=nt(1);
    elseif event==2,%departure of a call
        totalenergy=totalenergy+sum(Q)*(nt(2)-t)*850*signaling*trafficp/10*54+(laston*(850*cooling)+laston*850*signaling*controlp)*54*(nt(2)-t);
        maxenergy = maxenergy +sum(Q)*(nt(2)-t)*850*signaling*trafficp/10*54+(49*(850*cooling)+49*850*signaling*controlp)*54*(nt(2)-t);
        i=1;
        ran=rand();
        while ran>total(1,i,Q)/sum(Q),
            i=i+1;
        end
        Q(i)=Q(i)-1;
        
        t=nt(2);
    else %handover of a call from one cell to one of its neighbor
        totalenergy=totalenergy+sum(Q)*(nt(3)-t)*850*signaling*trafficp/10*54+(laston*(850*cooling)+laston*850*signaling*controlp)*54*(nt(3)-t);
        maxenergy = maxenergy +sum(Q)*(nt(3)-t)*850*signaling*trafficp/10*54+(49*(850*cooling)+49*850*signaling*controlp)*54*(nt(3)-t);
        i=1;
        ran=rand();
        while ran>total(1,i,Q)/sum(Q),
            i=i+1;
        end
        
        count3=0;
        
        temp=zeros(1,6);
        j=1;
        m=1;
        while j<=6,
            if Status(Neis(i,j))~=0,
                count3=count3+1;
                temp(m)=Neis(i,j);
                m=m+1;
            end
            
            j=j+1;
        end
        if count3>0,
            Q(i)=Q(i)-1;
            j=1;
            ran=rand();
            while ran>j/count3,
                j=j+1;
            end
            
            if Q(temp(j))<K,
                Q(temp(j))=Q(temp(j))+1;
            else
                B(temp(j))=B(temp(j))+1;
            end
            
        end
        
        t=nt(3);
    end
    %re-arranging BS sleeping pattern after a certain period of time
    %(dynamic sleeping)
    if t-lasttime>10 && t>20,
        i=1;
        ran=rand();
        while ran>i/N,
            i=i+1;
        end
        for j=i: i+48,
            if j>49,
                k=j-49;
            else
                k=j;
            end       
            Pstatus = Status;
            count1=0;
            count2=0;
            temp=zeros(1,6);
            m=1;
            for l=1:6
                if Pstatus(Neis(k,l))~=0,
                    count1=count1+1;
                    temp(m)=Neis(k,l);
                    m=m+1;
                end
                if Q(Neis(k,l))==K,
                    count2=count2+1;
                end
            end   
            if Pstatus(k)==0,       
                if count1==0,
                    Status(k)=2;
                elseif count2>0,
                    Status(k)=3;
                end        
            else         
                if Q(k)<3&&count1~=0,
                    Status(k)=0;
                    qk = Q(k);
                    for an=1: qk             
                        l=1;
                        ran=rand();
                        while ran>l/count1,
                            l=l+1;
                        end
                        if Q(temp(l))<K,
                            Q(temp(l))=Q(temp(l))+1;
                        else
                            B(temp(l))=B(temp(l))+1;
                        end
                    end
                    Q(k)=0;
                    
                end
            end
            
        end
        count=0;
        for i=1:N,
            if Status(i)~=0,
                count=count+1;
            end
        end
        laston =count;
        lasttime =t;
        
    end

end


savings = (maxenergy-totalenergy)/maxenergy;
totalb=total(1,N,B);
totalc=sum(A);
end