function [totalb,totalc,savings,totalenergy,maxenergy] = testenergysavings(trafficp)

%Discrete event simulation on 49-cell wrapped-around model to test energy
%savings under fixed BS (base station) sleeping scheme;
%Adjust the input parameter "trafficp" to see the impacts of different
%assumptions that how much energy consumption is dependent on carried
%traffic in BSs

%Power consumption of a BS can be broken into three parts:
%1. basic operation of a BS, such as air conditioning and
%power supply, consumed when the BS is active
%2. signal processing to coordinate other BS in the network, consumed when
%the BS is active
%3. power amplifier to connect to the mobile users, consumed proportionally
%with carried traffic of the BS

%output parameters
%totalb: number of blocked calls
%totalc: number of calls initiated
%savings (= totalenergy/maxenergy): relative energy savings as compared to the base case where no BS
%sleeping is implemented
%lambda_max: peak arrival rate
%mu: service rate
%delta: handover rate
signaling = 0.5; %proportion of energy consumption by power amplifier AND signal processing in total energy consumption of a BS
cooling = 1-signaling; %proportion of energy consumption by basic operations in total energy consumption of a BS
controlp = 1-trafficp; %proportion of energy consumption by signal processing in "signaling" (while trafficp is the proportion of energy consumption by power amplifier in "signaling")


[a,b]=textread('lambda.dat','%f %f'); %read the traffic profile, replace the file to get results under different traffic profiles
totalenergy=0;
maxenergy=0;
on=49;

count1=0;
count2=0;
count3=0;

N=49;
K=20;
Q = zeros(N,1);
A = zeros(N,1);
B = zeros(N,1);


p=1/6;
Neis=neighbors();
x=4;
z=1;
y=30;
threshold = 12; %controlling the time to switch off/on base stations under fixed switching scheme

% lastt=0;
t=0;
while t<1600
    lasty = y;
    i=1;
    while i<length(a),
        if t>=a(i)/6.2832*1600&t<a(i+1)/6.2832*1600,
            y=30*((b(i+1)-b(i))/(a(i+1)-a(i))*t*6.2832/1600+(a(i+1)*b(i)-a(i)*b(i+1))/(a(i+1)-a(i)));
            break;
        end
        i=i+1;
    end

    
    r(1)=-1/y/N*log(1-rand());
    
    nt(1)=t+r(1);
    
    if sum(Q)>0,
        
        r(2)=-1/x/sum(Q)*log(1-rand());
        
        nt(2)=t+r(2);
        
        
        r(3)=-1/z/sum(Q)*log(1-rand());
        
        nt(3)=t+r(3);
        
    else
        
        r(2)=-1/x*log(1-rand());
        
        nt(2)=nt(1)+r(2);
        
        r(3)=-1/z*log(1-rand());
        
        nt(3)=nt(1)+r(3);
        
    end
    
    event=min(nt);
    
    
    if event==1,%arrival
        
        totalenergy=totalenergy+sum(Q)*(nt(1)-t)*850*signaling*trafficp/10*54+(on*(850*cooling)+on*850*signaling*controlp)*54*(nt(1)-t);
        maxenergy=maxenergy+sum(Q)*(nt(1)-t)*850*signaling*trafficp/10*54+(49*(850*cooling)+49*850*signaling*controlp)*54*(nt(1)-t);


        i=1;
        ran=rand();
        while ran>i/N,
            i=i+1;
        end
        
        l=i;
        if y<threshold,
            
            count3=count3+1;
            if i==1 || i==2 || i==9 || i==16 || i==15 || i==14,
                l=8;
            elseif i==44 || i==45 || i==7 || i==13 || i==12 || i==5,
                l=6;
            elseif i==10 || i==11 || i==19 || i==25 || i==24 || i==17,
                l=18;
            elseif i==20 || i==21 || i==28 || i==35 || i==34 || i==26,
                l=27;
            elseif i==22 || i==23 || i==31 || i==37 || i==36 || i==29,
                l=30;
            elseif i==32 || i==33 || i==40 || i==47 || i==46 || i==38,
                l=39;
            elseif i==41 || i==42 || i==48 || i==3 || i==4 || i==43,
                l=49;
            end
            
            for i=1:N,
                if i~=8 && i~=6 && i~=18 && i~=27 &&i~=30 &&i~=39 &&i~=49,
                    count1=0;
                    temp=zeros(1,6);
                    k=1;
                    m=1;
                    while k<=6,
                        if Neis(i,k)==8,
                            count1=count1+1;
                            temp(m)=Neis(i,k);
                            m=m+1;
                        elseif Neis(i,k)==6,
                            count1=count1+1;
                            temp(m)=Neis(i,k);
                            m=m+1;
                        elseif Neis(i,k)==18,
                            count1=count1+1;
                            temp(m)=Neis(i,k);
                            m=m+1;
                        elseif Neis(i,k)==27,
                            count1=count1+1;
                            temp(m)=Neis(i,k);
                            m=m+1;
                        elseif Neis(i,k)==30,
                            count1=count1+1;
                            temp(m)=Neis(i,k);
                            m=m+1;
                        elseif Neis(i,k)==39,
                            count1=count1+1;
                            temp(m)=Neis(i,k);
                            m=m+1;
                        elseif Neis(i,k)==49,
                            count1=count1+1;
                            temp(m)=Neis(i,k);
                            m=m+1;
                        end
                        k=k+1;
                    end
                    if Q(i)>0,
                        for j=1: Q(i),
                            
                            
                            k=1;
                            ran=rand();
                            while ran>k/count1,
                                k=k+1;
                            end
                            if Q(temp(k))<K,
                                Q(temp(k))=Q(temp(k))+1;
                            else
                                B(temp(k))=B(temp(k))+1;
                            end
                        end
                        Q(i)=0;
                    end
                end
                
            end
            
        end
        
        A(l)=A(l)+1;
        if Q(l)<K,
            Q(l)=Q(l)+1;
        else
            B(l)=B(l)+1;
        end
        
        t=nt(1);
        
    elseif event==2,%departure
        
        totalenergy=totalenergy+sum(Q)*(nt(2)-t)*850*signaling*trafficp/10*54+(on*(850*cooling)+on*850*signaling*controlp)*54*(nt(2)-t);
        maxenergy=maxenergy+sum(Q)*(nt(2)-t)*850*signaling*trafficp/10*54+(49*(850*cooling)+49*850*signaling*controlp)*54*(nt(2)-t);
        if y>=threshold,
            on=49;
        else
            on=7;
        end
        i=1;
        ran=rand();
        while ran>total(1,i,Q)/sum(Q),
            i=i+1;
        end
        
        Q(i)=Q(i)-1;
        
        t=nt(2);
        
    else %handover
        
        totalenergy=totalenergy+sum(Q)*(nt(3)-t)*850*signaling*trafficp/10*54+(on*(850*cooling)+on*850*signaling*controlp)*54*(nt(3)-t);
        maxenergy=maxenergy+sum(Q)*(nt(3)-t)*850*signaling*trafficp/10*54+(49*(850*cooling)+49*850*signaling*controlp)*54*(nt(3)-t);
        if y>=threshold,
            on=49;
        else
            on=7;
        end
        count2=count2+1;
        i=1;
        ran=rand();
        while ran>total(1,i,Q)/sum(Q),
            i=i+1;
        end
        
        if y>threshold,
            Q(i)=Q(i)-1;
            count2=count2+1;
            j=1;
            ran=rand();
            while ran>j*p,
                j=j+1;
            end
            if Q(Neis(i,j))<K,
                Q(Neis(i,j))=Q(Neis(i,j))+1;
            else
                B(Neis(i,j))=B(Neis(i,j))+1;
            end
        end
        
        t=nt(3);
        
        
    end
    if lasty>=threshold && y < threshold %check current time to determine which state the BSs should be in
        on = 7;
        for i = 1:49
            if i==1 || i==2 || i==9 || i==16 || i==15 || i==14,
                l=8;
            elseif i==44 || i==45 || i==7 || i==13 || i==12 || i==5,
                l=6;
            elseif i==10 || i==11 || i==19 || i==25 || i==24 || i==17,
                l=18;
            elseif i==20 || i==21 || i==28 || i==35 || i==34 || i==26,
                l=27;
            elseif i==22 || i==23 || i==31 || i==37 || i==36 || i==29,
                l=30;
            elseif i==32 || i==33 || i==40 || i==47 || i==46 || i==38,
                l=39;
            elseif i==41 || i==42 || i==48 || i==3 || i==4 || i==43,
                l=49;
            else
                l=-1;
            end
        end
        if l>0
            for j=1:Q(i)
                if Q(l) < K
                    Q(l) = Q(l) +1;
                else
                    B(l) = B(l) +1;
                end
            end
            Q(i)=0;
        end
    elseif lasty<threshold && y >= threshold
        on = 49;
    end

end

savings = (maxenergy-totalenergy)/maxenergy;
totalb=total(1,N,B);
totalc=sum(A);
end
